@@ -1,6 +1,6 @@
 from functools import wraps
 from tryhaskell import TryHaskell
-from helga.plugins import match
+from helga.plugins import command
 
 
 def clean_output(f):
@@ -19,8 +19,10 @@ def show_type(result):
     return ' :: '.join([result.expr.strip(), result.type])
 
 
-def show_value(result):
+def show_value(result, display_errors):
     if not result.ok:
+        if not display_errors:
+            return None
         # Reduce multiple white spaces with a single space.
         msg = ' '.join(result.value.split())
         return 'ERROR: ' + msg
@@ -31,14 +33,14 @@ def show_value(result):
     return show_type(result)
 
 
-@match(r'^(\:t |>\s*\:t |>).*$')
+@command('haskell', aliases=['h'],
+         help='Run haskell expressions. Usage: helga h(askell) (:t) <expression>')
 @clean_output
-def haskell(client, channel, nick, message, matches):
-    # Simplify checking matches.
-    matches = ''.join(matches)
-    if ':t' in matches:
+def haskell(client, channel, nick, message, cmd, args):
+    # Only show errors when using the explicit !haskell command.
+    display_errors = cmd == 'haskell'
+    if ':t' in args[:1]:
         _, exp = message.split(':t', 1)
         r = TryHaskell.get(exp)
-        return show_type(r) if r.ok else show_value(r)
-    elif '>' in matches:
-        return show_value(TryHaskell.get(message[1:]))
+        return show_type(r) if r.ok else show_value(r, display_errors)
+    return show_value(TryHaskell.get(' '.join(args)), display_errors)
